//------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"
//------------------------------------------------------------------

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 DeepSkyBlue
#property indicator_color2 PaleVioletRed
#property indicator_color3 DeepSkyBlue
#property indicator_color4 PaleVioletRed


#define PRICE_HA 7

extern int  LongPeriod           = 55;
extern int  LongPrice            = PRICE_HA;
extern int  ShortPeriod          = 55;
extern int  ShortPrice           = PRICE_TYPICAL;
extern bool LinesVisible         = true;
extern bool ColorCandles         = true;

extern bool   ShowArrows         = true;
extern color  arrowsUpColor      = DeepSkyBlue;
extern color  arrowsDnColor      = Red;
extern string arrowsIdentifier   = "ZLT_Arrow";
extern double arrowDistance      = 2.0;

double long[];
double short[];
double histU[];
double histD[];


int init() {
   int styleline  = STYLE_DOT;
   int style      = DRAW_NONE;         if(LinesVisible)  { style = DRAW_LINE; styleline = STYLE_SOLID; }
   int stylebar   = DRAW_HISTOGRAM;    if(!ColorCandles) { stylebar = DRAW_NONE; }
   
   SetIndexBuffer(0,histU); SetIndexStyle(0,stylebar);
   SetIndexBuffer(1,histD); SetIndexStyle(1,stylebar);

   SetIndexBuffer(2,long);  SetIndexStyle(2,style,styleline); SetIndexLabel(2,"ZL TEMA "+LongPeriod);
   SetIndexBuffer(3,short); SetIndexStyle(3,style,styleline); SetIndexLabel(3,"ZL TEMA "+ShortPeriod);

   IndicatorShortName("ZeroLag TEMA ("+LongPeriod+","+ShortPeriod+")");
   return(0);
}

int deinit() { 
   if (ShowArrows) deleteArrows();
   return(0); 
}


int start() {
   int counted_bars = IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars > 0) counted_bars--;
           int limit = Bars-counted_bars;
   
   for (int i=limit; i>=0; i--)
   {  
      if (LongPeriod > 1)
      {
         double tema10  = iTema(getPrice(LongPrice,i),LongPeriod,i,0);
         double tema11  = iTema(tema10               ,LongPeriod,i,1);
                long[i] = 2.0*tema10-tema11; 
      }
      if (ShortPeriod > 1)
      {
         double tema20   = iTema(getPrice(ShortPrice,i),ShortPeriod,i,2);
         double tema21   = iTema(tema20                ,ShortPeriod,i,3);
                short[i] = 2.0*tema20-tema21; 
      }
      if (LongPeriod > 1 && ShortPeriod>1)
      {
         histU[i] = EMPTY_VALUE;
         histD[i] = EMPTY_VALUE;
               if (long[i]>short[i]) { histU[i] = Low[i]; histD[i] = High[i]; }
               if (long[i]<short[i]) { histU[i] = High[i]; histD[i] = Low[i]; }
      }
      if (ShowArrows) manageArrow(i);      
   }         
   return(0);
}

//-------------------------------------------------------------------------------------------------

void manageArrow(int i) {

   deleteArrow(Time[i]);
   
   if(long[i]>short[i] && long[i+1]<short[i+1]) { drawArrow(i,arrowsDnColor,234,true);  }
   if(long[i]<short[i] && long[i+1]>short[i+1]) { drawArrow(i,arrowsUpColor,233,false); }
   
   return(0);     
}              

void drawArrow(int i,color theColor,int theCode,bool up) {
   string name = arrowsIdentifier+":"+Time[i];
   //double gap  = arrowDistance*iATR(NULL,0,20,i)/4.0;   
   double gapup  = arrowDistance*iATR(NULL,0,20,i)/3.0;   
   double gapdn  = arrowDistance*iATR(NULL,0,20,i)/12.0;   
   
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+gapup);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gapdn);
}

void deleteArrows() {
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--) {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

void deleteArrow(datetime time) {
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}

//----------------------------------------------------------------------------------------------------

double workHa[][4];

double getPrice(int usePrice, int i) {
   if (ArraySize(workHa)!= Bars) ArrayResize(workHa,Bars); int r = Bars-i-1;
   
   if (usePrice==PRICE_HA)
   {   
      double haOpen  = (workHa[r-1][2] + workHa[r-1][3])/2.0;
      double haClose = (Open[i] + High[i] + Low[i] + Close[i]) / 4.0;
      double haHigh  = MathMax(High[i], MathMax(haOpen,haClose));
      double haLow   = MathMin(Low[i] , MathMin(haOpen,haClose));

      if(haOpen  <haClose) { workHa[r][0] = haLow;  workHa[r][1] = haHigh; } 
      else                 { workHa[r][0] = haHigh; workHa[r][1] = haLow;  } 
                             workHa[r][2] = haOpen;
                             workHa[r][3] = haClose;
      return((workHa[r][0] + workHa[r][1] + workHa[r][2] + workHa[r][3])/4.0);
   }
   else return(iMA(NULL,0,1,0,MODE_SMA,usePrice,i));
}

double workTema[][12];
#define _ema1 0
#define _ema2 1
#define _ema3 2

double iTema(double price, double period, int i, int instanceNo=0) {
   if (ArraySize(workTema)!= Bars) ArrayResize(workTema,Bars); int r=Bars-i-1; instanceNo*=3;

   double alpha = 2.0 / (1.0+period);
          workTema[r][_ema1+instanceNo] = workTema[r-1][_ema1+instanceNo]+alpha*(price                        -workTema[r-1][_ema1+instanceNo]);
          workTema[r][_ema2+instanceNo] = workTema[r-1][_ema2+instanceNo]+alpha*(workTema[r][_ema1+instanceNo]-workTema[r-1][_ema2+instanceNo]);
          workTema[r][_ema3+instanceNo] = workTema[r-1][_ema3+instanceNo]+alpha*(workTema[r][_ema2+instanceNo]-workTema[r-1][_ema3+instanceNo]);
   return(workTema[r][_ema3+instanceNo]+3.0*(workTema[r][_ema1+instanceNo]-workTema[r][_ema2+instanceNo]));
}